from qgis.core import *
from qgis.gui import *
import numpy as np
from cmath import rect, phase

@qgsfunction(args=3, group='Custom')

def zigzag(value1 , feature, parent):
   """
      transforme une geometrie lineaire en zigzag
      <h4>Syntax</h4>
      <pre>zigzag(geom, longueur, amplitude)</pre>
 
      <h4>Exemple</h4>
      zigzag($geometry,200,100)
   
   """

   def meanAngle(a1, a2):
      return phase((rect(1, a1) + rect(1, a2)) / 2.0)

   # Number of zigzag segments
   geom=value1[0]
   wavelength=value1[1]
   amplitude=value1[2]
	
   length = geom.length()
   segments = np.round(length / wavelength)
   # Find equally spaced points that approximate the line
   points = [geom.interpolate(distance).asPoint() for distance in np.linspace(0, length, segments)]

   # Calculate the azimuths of the approximating line segments
   azimuths = np.radians([points[i].azimuth(points[i + 1]) for i in range(len(points) - 1)])

   # Average consecutive azimuths and rotate 90 deg counterclockwise
   zigzagazimuths = [azimuths[0] - np.pi / 2]
   zigzagazimuths.extend([meanAngle(azimuths[i], azimuths[i - 1]) - np.pi / 2 for i in range(len(points) - 1)])
   zigzagazimuths.append(azimuths[-1] - np.pi / 2)

   # Offset the points along the zigzagazimuths
   zigzagpoints = []
   for i in range(len(points)):
      # Alternate the sign
      dst = amplitude * (1 - 2 * np.mod(i, 2))
      zigzagpoints.append(QgsPoint(points[i][0] + np.sin(zigzagazimuths[i]) * dst, points[i][1] + np.cos(zigzagazimuths[i]) * dst))
   # Create new feature from the list of zigzag points
   gLine = QgsGeometry.fromPolyline(zigzagpoints)
	
   return gLine
